// ref https://github.com/vuejs/vitepress/blob/main/src/node/markdown/plugins/highlight.ts
const escapeHtml = require("escape-html");
const prism = require("prismjs");
const chalk = require("chalk");

// prism is listed as actual dep so it's ok to require
const loadLanguages = require("prismjs/components/index");

// required to make embedded highlighting work...
loadLanguages(["markup", "css", "javascript"]);

function wrap(code, lang) {
  if (lang === "text") {
    code = escapeHtml(code);
  }
  return { code, lang };
}

module.exports = (str, lang) => {
  if (!lang) {
    return wrap(str, "text");
  }
  lang = lang.toLowerCase();
  const rawLang = lang;
  if (lang === "vue" || lang === "html") {
    lang = "markup";
  }
  if (lang === "md") {
    lang = "markdown";
  }
  if (lang === "ts") {
    lang = "typescript";
  }
  if (lang === "py") {
    lang = "python";
  }
  if (!prism.languages[lang]) {
    try {
      loadLanguages([lang]);
    } catch (e) {
      // eslint-disable-next-line no-console
      console.warn(
        chalk.yellow(
          `[vitepress] Syntax highlight for language "${lang}" is not supported.`
        )
      );
    }
  }
  if (prism.languages[lang]) {
    const code = prism.highlight(str, prism.languages[lang], lang);
    return wrap(code, rawLang);
  }
  return wrap(str, "text");
};
